import collections


class Components(object):
    """
    List of the components, their individual (hard coded) mass, the number of
    each component and the resulting total mass of the program.
    """
    _weights = collections.OrderedDict([
        ("constant", 1),
        ("binding", 1),
        ("invocation", 3),
        ("conditional", 4),
        ("loop", 5),
        ("assignment", 6)
    ])

    def __init__(self):
        for field in self._weights.keys():
            setattr(self, field + "s", 0)

    def create_table(self):
        return "\n".join(
            [self._row("Components", "Mass", "Count")] +
            [self._row(component, weight, self._count(component))
                for component, weight in self._weights.items()]
        ) + "\n"

    def _count(self, field):
        return getattr(self, field + "s")

    def _row(self, component, mass, count):
        return "| " + self._align(component) + \
            " | " + self._center(mass) + \
            " | " + self._center(count) + " |"

    def _align(self, field):
        return self._pad(field, 0, 11 - len(field))

    def _center(self, value):
        field = str(value)
        total = 5 - len(field)
        left = total // 2
        right = total - left
        return self._pad(field, left, right)

    def _pad(self, field, left, right):
        return " " * left + field + " " * right

    def calc_mass(self):
        return sum(
            [self._count(component) * weight
                for component, weight in self._weights.items()]
        )
